/*
   WP-TOOL for Linux FBB
   Copyright (c) 2000..2002 by Stewart Wilkinson G0LGS
*/

#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <stdarg.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
#include <getopt.h>
#include <pwd.h>
#include <sys/signal.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>

#include "wplist.h"
#include "wputils.h"
#include "utils.h"

extern char *optarg;
extern int optind;
extern int opterr;
extern int getopt(int argc, char * const *argv, const char *optstring);

char system_dir[256];
char wp_sys[256];
char wp_old[256];
char myconfig[128];
int uk_only=0;
int non_uk = 0;
int quiet = 0;
int keep = 0;
int verbose = 0;
int bad_call = 0;
int bad_ha = 0;
int sel_call = 0;
int sel_ha = 0;
int expired = 0;
int future = 0;
long record = 0;
time_t time_now;
time_t time_expire;
WP_Rec Rec;
FILE *fpti;
FILE *fpto;
#define BUFSIZE 16384
static char buffer[BUFSIZE];
struct srch *call_list = (NULL);
struct srch *route_list = (NULL);

#if V700
int ReadInit()
{
   #define SYSTEM_DIR 5

   int index = 0;
   int end = 0;
   FILE *fptr;
   char buffer[256];

   if ((fptr = fopen ("init.srv", "r")) != NULL)
   {
      while( !end && fgets(buffer, sizeof(buffer), fptr) ){

         /* Comments ignored */
         if (*buffer == '#')
            continue;

         switch (++index) {

            case SYSTEM_DIR :
               sscanf(buffer, "%[^\n]\n", system_dir);
               end=1;
               break;

         }

      }
      fclose(fptr);
  
      sprintf( wp_sys, "%swp/wp.sys", system_dir );
      sprintf( wp_old, "%swp/wp.bak", system_dir );

   }else{
      fprintf( stderr, "Unable to open 'init.srv'\n" );
      return 1;
   }
   return 0;
}
#else

int ReadInit()
{
   #define  FBBCONF "/etc/ax25/fbb.conf"

   int len, i;
   int end=0;
   FILE *fptr;
   char buffer[256];
   char *scan;
   char *ptr;
   char key[20];
   char val[256];
   char pathname[256];

   if( (ptr = getenv("FBBCONF")) == NULL ){
     strcpy( pathname, FBBCONF);
   }else{
     strcpy( pathname, ptr);
   }

   if ((fptr = fopen (pathname, "r")) != NULL)
   {
      while( !end && fgets(buffer, sizeof(buffer), fptr) ){

         scan = buffer;

         /* Eat Spaces */
         while (isspace(*scan))
            ++scan;

         /* Comments & Blanks ignored */
         if (*scan == '#' || *scan == '\0' )
            continue;

         /* remove leading \n */
         len = strlen(scan);
         if( len && (scan[len-1] == '\n'))
            scan[len-1] = '\0';

         /* keyword */
         i=0;
         ptr = key;
         while( isgraph(*scan)) {
            if( i++ < 4 ) {
               *ptr++ = *scan;
            }
            ++scan;
            if(*scan == '=')
               break;         
         }
         *ptr = '\0';

         /* find the '=' */
         scan = strchr(scan, '=');

         /* copy value */
         if(scan){
            ++scan;
            while( isspace(*scan))
               ++scan;

            /* if 1st char is '"' then string */
            if( *scan == '"' ){
               ++scan;
               ptr = strrchr(scan,'"');
               if(ptr)
                  *ptr = '\0';
            }

            strcpy(val, scan);            

         }else{
            strcpy(val, "");
         }

         if( strcmp(key, "data") == 0 ){
            sprintf( wp_sys, "%s/wp/wp.sys", scan );
            sprintf( wp_old, "%s/wp/wp.bak", scan );
         }

         if( strcmp(key, "conf") == 0 ){
            sprintf( myconfig, "%s/wpclean.cfg", scan );
         }

      }

      fclose(fptr);
  
   }else{
      fprintf( stderr, "Unable to open '%s'\n", pathname );
      return 1;
   }
   return 0;
}
#endif

int copyfile (char *old, char *new)
{
   int src;
   int dest;
   int num_read;
   int num_writ;
   int ret = 1;

   if( (src = open(old, O_RDONLY | O_BINARY, S_IREAD | S_IWRITE ) ) == EOF ){
      fprintf( stderr, "Unable to find %s\n", old );
      return 0;
   }

   if( (dest = open(new, O_WRONLY | O_CREAT | O_TRUNC | O_BINARY, S_IREAD | S_IWRITE ) ) == EOF ){
      fprintf( stderr, "Unable to Create %s\n", new );
      close(src);
      return 0;
   }

   for (;;){
      num_read = read( src, buffer, BUFSIZE );

      if( num_read == -1 ){
         ret = 0;
         break;
      }else if( num_read == 0) {
         ret = 1;
         break;
      }

      num_writ = write(dest, buffer, num_read);

      if( num_writ != num_read ){
         ret = 0;
         break;
      }

   }
   close(src);
   close(dest);

   return ret;
}

void header(char *s)
{
   fprintf( stdout, "\n%s " Version "\n", s);
   fprintf( stdout, "Copyright (c) 2000..2002 by Stewart Wilkinson (G0LGS)\n\n");
}

void warn(FILE *s)
{
   fprintf( s, "\n=====================================================================\n" );
   fprintf( s, "WARNING:\tAttempting to clean the WP Database whilst XFBB\n\t\tis active may crash the system.\n" );
   fprintf( s, "=====================================================================\n" );
}

void usage(char *s)
{
   header(s);
   fprintf( stdout, "Usage: %s [Options]\n\n", s );
   fprintf( stdout, "\t-q call[,call] ... | --query=call[,call] ... Show Selected Entry(s)\n" );
   fprintf( stdout, "\t--clean | --clean=days Clear Old/Invalid Entries\n");
   fprintf( stdout, "\t-L | --list  List ALL Entries\n" );
   fprintf( stdout, "\t-v | --verbose\n" );
   fprintf( stdout, "\t-Q | --quiet\n" );
   fprintf( stdout, "\t--uk-only\tRemove ALL non-UK calls (requires --clean)\n" );
   warn( stdout );
}

static void sig_func (int sig)
{
   int pid, pstatus;

   sig &= 0xff;
   pid = wait(&pstatus);
   signal( sig, sig_func );

   switch( sig ){

   case SIGINT:
        fflush( stdout );
        fflush( stderr );
        fprintf( stderr, "\r%-80s\n\n", "Aborted by User");

        Free_Search_List(&call_list);
        Free_Search_List(&route_list);

        if( fpti != NULL )
           fclose(fpti);

        if( fpto != NULL )
           fclose(fpto);

        exit(255);
	break;
   }
}

int ReadCfg(void)
{

   FILE *f;
   char buffer[255];
   char param[128];
   int line = 0;
   int calls = 0;
   int routes = 0;

   if(!(f=fopen( myconfig , "r"))){
      return -1;
   }

   while(fgets(buffer,sizeof(buffer),f)){

      line++;

      if( strncasecmp("! calls", buffer, 7) ==0 ) {
         calls=1;
         routes=0;

      }else if( strncasecmp("! routes", buffer, 8) ==0 ) {
         calls=0;
         routes=1;

      }else if( strncasecmp("! END", buffer, 5) ==0 ) {
         calls=0;
         routes=0;
         break;

      }else if( ( *buffer == '#' ) || ( *buffer == ';' ) || (*buffer == '\n') ){
         continue;

      }else{
         sscanf( buffer, "%s ", param );
         if(calls) {
            Add_Search_Item(&call_list, param);
         }else if(routes) {
            Add_Search_Item(&route_list, param);
         }
      }
   }
   fclose(f);

   return 0;
}

char *_date( long temp )
{
   struct tm *sdate;
   static char cdate[40];

   sdate = localtime (&temp);

   sprintf (cdate, "%02d%02d%02d", sdate->tm_year % 100, sdate->tm_mon + 1, sdate->tm_mday);

   return (cdate);
}


int Check_Record( WP_Rec * Rec )
{
   Rec->Callsign[6] = '\0';
   Rec->Name[12] = '\0';
   Rec->Active_Homebbs[40] = '\0';
   Rec->Temp_Homebbs[40] = '\0';
   Rec->Active_Zip[8] = '\0';
   Rec->Temp_Zip[8] = '\0';
   Rec->Active_Qth[30] = '\0';
   Rec->Temp_Qth[30] = '\0';

   if((Check_Call(Rec->Callsign) != 0)) {
      if( verbose ){
         fprintf( stdout, "Bad Callsign: %s\n", Rec->Callsign );
      }
      bad_call++;
      return(1);
   }
   
   if((Check_Home(Rec->Active_Homebbs) != 0)) {

      if((Check_Home(Rec->Temp_Homebbs) != 0)) {
         if( verbose ){
            fprintf( stdout, "Bad HomeBBS: %s for %s\n", Rec->Active_Homebbs, Rec->Callsign );
         }
         bad_ha++;
         return(2);

      }else{
         strcpy(Rec->Active_Homebbs, Rec->Temp_Homebbs);
      }
   }


   if( Match_Item(&call_list, Rec->Callsign) == 0 ) {
      sel_call++;
      return(3);

   }else if( Match_Item(&route_list, Rec->Active_Homebbs) == 0 ) {
      sel_ha++;
      return(4);

   }else if( Rec->Last_Modif > time_now ) {
      future++;
      return(5);

   }else if( Rec->Last_Seen > time_now ) {
      future++;
      return(5);

   }else if( Rec->Last_Seen < time_expire ) {
      expired++;
      return(7);

   }else if( uk_only ) {
      if( (strncasecmp( Rec->Callsign, "G", 1 ) != 0) &&
          (strncasecmp( Rec->Callsign, "M", 1 ) != 0) && 
          (strncasecmp( Rec->Callsign, "2", 1 ) != 0) ) {

         non_uk++;
         return(8);
      }

   }

   return(0);
}

int Clean_WP( int clean )
{
   int pid;
   int delay;
   int v;

   /* read the config file */
   if( ReadCfg() != 0 ){
      fprintf( stderr, "ERROR: Unable to open/process '%s'\n", myconfig );
      return(1);
   }
 
   /* Try and warn if xfbb or xfbbd active */
   pid = WEXITSTATUS(system("pidof xfbb >/dev/null 2>&1"));

   if( (pid != -1) && (pid != 0) ){
      pid = WEXITSTATUS(system("pidof xfbbd >/dev/null 2>&1"));
   }

   if( (pid == 127) ){
      fprintf( stderr, "\007Unable to determine if FBB is active.\n" );
      exit(1);
   }

   if( (pid == 0) ){
      warn( stderr );
      fprintf( stderr, "\007" );
      delay=30;
      while(delay){
         fprintf( stderr, "\rPress CTRL-C in %d Secs to Cancel  \r", delay);
         delay--;
         v=sleep(1);
      }
      fprintf( stderr, "\r                                    \r");
   }

   if (!copyfile (wp_sys, wp_old)) {
      unlink (wp_old);
      return(1);
   }

   if(!quiet){
      fprintf( stdout, "\nBackup copy of '%s' saved as '%s'\n", wp_sys, wp_old );
   }

   if ((fpti = fopen (wp_old, "rb")) == NULL)
   {
      fprintf (stderr, "Cannot open '%s' \n", wp_old);
      return(2);
   }


   if ((fpto = fopen (wp_sys, "wb")) == NULL)
   {
      fprintf (stderr, "Cannot create '%s' \n", wp_sys);
      return(3);
   }


   time_now = time(NULL);
   time_expire = time_now - (time_now % 86400) - (clean * 86400);

   if( verbose ){
      Show_Search_List(&call_list, "Callsign");
      Show_Search_List(&route_list, "HA");
   }

   while( fread(&Rec, sizeof(WP_Rec), 1, fpti ))
   {
      record++;
      if( Check_Record( &Rec ) == 0 ){  

         /* Keep Record */
         keep++;

         if (fwrite (&Rec, sizeof (WP_Rec), 1, fpto) == 0){
            fprintf (stderr, "Error Writing to '%s' \n", wp_sys);
            return(1);
         }

      }
   }

   if(! quiet)
      fprintf( stdout, "\n" );

   Free_Search_List(&call_list);
   Free_Search_List(&route_list);

   if(!quiet){
      fprintf( stdout, "\nFound %d Bad Callsigns, %d Bad HA's, %d Future, %d Expired, %d Selected\n", bad_call, bad_ha, future, expired, sel_call+sel_ha );
      if( uk_only ){
         fprintf( stdout, "Found %d non-uk calls\n", non_uk );
      }
      fprintf( stdout, "%d of %ld Records Deleted\n\n", bad_call+bad_ha+future+expired+sel_call+sel_ha+non_uk, record );
   }

   fclose(fpti);
   fclose(fpto);
   return(0);
}

int find( WP_Rec * Rec, char *call )
{
   Rec->Callsign[6] = '\0';
   Rec->Name[12] = '\0';
   Rec->Active_Homebbs[40] = '\0';
   Rec->Temp_Homebbs[40] = '\0';
   Rec->Active_Zip[8] = '\0';
   Rec->Temp_Zip[8] = '\0';
   Rec->Active_Qth[30] = '\0';
   Rec->Temp_Qth[30] = '\0';

   return( strcasecmp(Rec->Callsign, call) != 0 );
}

void PrintRec(WP_Rec * Rec )
{
   char modif[10];
   char seen[10];

   strcpy(modif, _date(Rec->Last_Modif) );
   strcpy(seen, _date(Rec->Last_Seen) );

   fprintf( stdout, "\t%-4ld: %s %s %-6s (%u) %s\n", 
    record,
    modif,
    seen,
    Rec->Callsign,
    Rec->Seen,
    (*Rec->Name) ? Rec->Name : "?" );

   fprintf( stdout, "\t[1] %s", Rec->Active_Homebbs );
   fprintf( stdout, " Zip:%s", (*Rec->Active_Zip) ? Rec->Active_Zip : "?" );
   fprintf( stdout, " Qth:%s\n", (*Rec->Active_Qth) ? Rec->Active_Qth : "?" );

   fprintf( stdout, "\t[2] %s", Rec->Temp_Homebbs );
   fprintf( stdout, " Zip:%s", (*Rec->Temp_Zip) ? Rec->Temp_Zip : "?" );
   fprintf( stdout, " Qth:%s\n", (*Rec->Temp_Qth) ? Rec->Temp_Qth : "?" );

   fprintf( stdout, "\n" );
}

int Query(char *call)
{
   int found =0;

   if ((fpti = fopen (wp_sys, "rb")) == NULL)
   {
      fprintf (stderr, "Cannot open '%s' \n", wp_sys);
      return(2);
   }

   record=0;
   while( fread(&Rec, sizeof(WP_Rec), 1, fpti ))
   {
      if( (find(&Rec, call) ==0) ){  
         found++;
         fprintf( stdout, "\r%-50s\r", " " );
         PrintRec( &Rec );

         /* Jump to End to end Search */
         fseek(fpti, 0, SEEK_END);
      }
      record++;
   }

   fclose(fpti);

   if( ! found ){
      fprintf( stdout, "No Record Found for '%s'\n", call );
   }
   return(0);
}

int List()
{
   if ((fpti = fopen (wp_sys, "rb")) == NULL)
   {
      fprintf (stderr, "Cannot open '%s' \n", wp_sys);
      return(2);
   }

   record=0;
   while( fread(&Rec, sizeof(WP_Rec), 1, fpti ))
   {
      find(&Rec, "");
      PrintRec( &Rec );
      record++;
   }

   fclose(fpti);
   return(0);
}

int main(int argc, char *argv[])
{
   char Cmd[255];
   char *ap;
   int ch;
   int clean = 0;
   int list = 0;
   int query = 0;
   int len = 0;
   int bad = 0;
   char *arg;
   int option_index = 0;
   int ret = 0;
   static struct option long_options[] =
   {
   {"clean", 2, 0, 0},
   {"help", 0, 0, 'h'},
   {"list", 0, 0, 'L'},
   {"query", 1, 0, 'q'},
   {"quiet", 0, 0, 'Q'},
   {"uk-only", 0, 0, 0},
   {"verbose", 0, 0, 'v'},
   {0, 0, 0, 0}
   };
   char qcalls[255];
   char *p, *p1;

   signal( SIGINT, sig_func );

   strcpy( Cmd, argv[0] );
   if( (ap = rindex( Cmd, '/') ) != NULL ){
     ap++;
   }else{
     ap=Cmd;
   }

   while((ch = getopt_long (argc, argv, "?hHLq:Qv", long_options, &option_index)) != (-1) ) {

      switch(ch) {

         case 0:
            if( strcmp(long_options[option_index].name,"clean") == 0 ){
               if(optarg) {
                  arg=optarg;
                  while(*arg){
                     if(!isdigit(*arg)){
                        fprintf( stderr, "clean value must be numeric\n" );
                        bad++;
                     }
                     arg++;
                     len++;
                  }
                  if(!bad && (len<=3)){
                     sscanf(optarg,"%d",&clean);
                     if( (clean<15) || (clean>180) ){
                        fprintf( stderr, "clean value out of range (15-180 days)\n" );
                        clean=0;
                     }
                  }
               }else{
                  clean = 90;
               }
            }else if( strcmp(long_options[option_index].name,"uk-only") == 0 ){
               uk_only=1;
            }

            break;

         case 'L':
            list = 1;
            break;

         case 'q':
         case 'u':
            strcpy(qcalls, optarg);
            strupr(qcalls);
            query = 1;
            break;

         case 'Q':
            verbose = 0;
            quiet = 1;
            break;

         case 'v':
            verbose=1;
            quiet=0;
            break;

         case '?':
         case 'h':
         case 'H':
            usage(ap);
            return 1;

         default:
            return 1;
      }
   }

/*
   if (optind < argc) {
      fprintf( stderr, "non-option ARGV-elements:" );
      while (optind < argc)
         fprintf( stderr, "%s ", argv[optind++]);
      fprintf( stderr, "\n");
   }
*/
   if (optind < argc) {
      fprintf( stderr, "Extra arguments ignored\n" );
   }

   if( ! clean && ! query && ! list ){
      fprintf( stderr, "Nothing to do - Not enough parameters\n" );
      usage(ap);
      return 1;
   }

   if(!quiet){
      header(ap);
   }

   if( ! ReadInit() ){
      ret = 1;
   }

   if( query ){
      p=qcalls;
      if( (p1=strchr(p,',')) != NULL ){
         strcat(qcalls,",");
         while( (p != NULL) && ((p1=strchr(p,',')) != NULL) ) {
            p1[0] = '\0';

            if ((Check_Call(p) != 0)) {
               fprintf( stderr, "Invalid callsign '%s'\n", p);
               ret |= 1;
            }else{
               ret |= Query(p);
            }
            p=p1+1;
         }
      }else{
         if ((Check_Call(qcalls) != 0)) {
            fprintf( stderr, "Invalid callsign '%s'\n", qcalls);
            ret |= 1;
         }else{
            ret |= Query(qcalls);
         }
      }

   }else if( clean ){
      ret |= Clean_WP( clean );

   }else if( list ){
      ret |= List();

   }

   if( !quiet )
      fprintf( stdout, "\n" );

   signal( SIGINT, SIG_DFL );
   return ret;
}

