/*
   wplist.c
   Copyright (c) 2000 by Stewart Wilkinson G0LGS
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#ifndef fnmatch
#include <fnmatch.h>
#endif

#include "wplist.h"

void Insert_WP_Item(struct wplist **list, struct wplist *n)
{
   struct wplist *p, *tmp;

   if (*list == NULL || strcasecmp(n->call, (*list)->call) < 0) {
      tmp = *list;
      *list = n;
      n->next = tmp;
   } else {
      for (p = *list; p->next != NULL; p = p->next)
         if (strcasecmp(n->call, p->next->call) < 0)
            break;

      tmp = p->next;
      p->next = n;
      n->next = tmp;
   }
}

int Add_WP_Item(struct wplist **list, char *callsign, char *hroute, int type)
{
   struct wplist *n;

   if ((n = calloc(1, sizeof(struct wplist))) == NULL ) {
      fprintf( stderr, "Add_WP_Item: calloc %d\n", errno );
      return -1;
   }
   n->call = strdup(callsign);
   n->ha = strdup(hroute);
   n->type = type;
   Insert_WP_Item(list, n);
   return 0;
}

void Free_WP_List(struct wplist **list)
{
   struct wplist *tmp;

   while (*list != NULL) {
      free((*list)->call);
      free((*list)->ha);
      tmp = *list;
      *list = (*list)->next;
      free(tmp);
   }
}

void Remove_WP_Item(struct wplist **list, char *s)
{
   struct wplist *p, *tmp;

   if(*list == NULL) {
      return;
   } else if(strcasecmp(s, (*list)->call) == 0) {
      free((*list)->call);
      free((*list)->ha);
      tmp=*list;
      *list=(*list)->next;
      free(tmp);
   } else {
      for (p = *list; p->next != NULL; p = p->next) {
         if (strcasecmp(s, p->next->call) == 0) { 
            free(p->next->call);
            free(p->next->ha);
            tmp=p->next;
            p->next=p->next->next;
            free(tmp);
            break;
         }     
      }
   }
}

void Show_WP_List(struct wplist **list)
{
   struct wplist *p;
   int count=0;

   if(*list == NULL) {
      fprintf( stdout, "List is Empty:\n" );
   } else {
      fprintf( stdout, "List Contains:\n" );
      for (p = *list; p != NULL; p = p->next){
         count++;
         fprintf( stdout, "%s@%s\n", p->call, p->ha );
      }  
      fprintf( stdout, "\n %d Records\n", count );
   }
}

void Insert_Search_Item(struct srch **list , struct srch *n)
{
   struct srch *p, *tmp;

   if (*list == NULL || strcasecmp(n->data, (*list)->data) < 0) {
      tmp = *list;
      *list = n;
      n->next = tmp;
   } else {
      for (p = *list; p->next != NULL; p = p->next)
         if (strcasecmp(n->data, p->next->data) < 0)
            break;

      tmp = p->next;
      p->next = n;
      n->next = tmp;
   }
}

int Add_Search_Item(struct srch **list , char *s)
{
   struct srch *n;

   if ((n = calloc(1, sizeof(struct srch))) == NULL ) {
      fprintf( stderr, "Add_Call_Item: calloc %d\n", errno );
      return -1;
   }
   n->data = strdup(s);
   Insert_Search_Item(list, n);
   return 0;
}

void Free_Search_List(struct srch **list)
{
   struct srch *tmp;

   while (*list != NULL) {
      free((*list)->data);
      tmp = *list;
      *list = (*list)->next;
      free(tmp);
   }
}

void Remove_Search_Item(struct srch **list, char *s)
{
   struct srch *p, *tmp;

   if(*list == NULL) {
      return;
   } else if(strcasecmp(s, (*list)->data) == 0) {
      free((*list)->data);
      tmp=*list;
      *list=(*list)->next;
      free(tmp);
   } else {
      for (p = *list; p->next != NULL; p = p->next) {
         if (strcasecmp(s, p->next->data) == 0) {
            free(p->next->data);
            tmp=p->next;
            p->next=p->next->next;
            free(tmp);
            break;
         }
      }
   }
}

void Show_Search_List(struct srch **list, char *s)
{
   struct srch *p;
   int count=0;

   if(*list == NULL) {
      fprintf( stdout, "%s Search list is Empty:\n", s );
   } else {
      fprintf( stdout, "Searching by %s for:\n", s );
      for (p = *list; p != NULL; p = p->next){
         count++;
         fprintf( stdout, "%s\n", p->data );
      }
      fprintf( stdout, "\n" );
   }
}

int Match_Item(struct srch **list, char *s)
{
   struct srch *p;
   int ret = -1;

   if(*list != NULL) {

      for (p = *list; p != NULL; p = p->next){
         if( fnmatch(p->data, s, 0) == 0 ){
            ret=0;
            break;
         }
      }
   }

   return ret;
}

